%define anolis_release 4

Summary: The Threading Building Blocks library abstracts low-level threading details
Name:    tbb
Version: 2020.3
Release: %{anolis_release}%{?dist}
License: ASL 2.0
URL:     http://threadingbuildingblocks.org/

Source0: https://github.com/intel/tbb/archive/v%{version}/%{name}-%{version}.tar.gz
# These three are downstream sources.
Source6: tbb.pc
Source7: tbbmalloc.pc
Source8: tbbmalloc_proxy.pc

# Don't snip -Wall from C++ flags.  Add -fno-strict-aliasing, as that
# uncovers some static-aliasing warnings.
Patch0: tbb-2019-dont-snip-Wall.patch

# Make attributes of aliases match those on the aliased function.
Patch1: tbb-2020-attributes.patch

# Fix test-thread-monitor, which had multiple bugs that could (and did, on
# ppc64le) result in a hang.
Patch2: tbb-2019-test-thread-monitor.patch

# Fix a test that builds a 4-thread barrier, but cannot guarantee that more
# than 2 threads will be available to use it.
Patch3: tbb-2019-test-task-scheduler-init.patch

# Fix ABI break resulting from tbb::empty_task being removed from libtbb.so's
# exported symbols
Patch4: tbb-mark-empty_task-execute-with-gnu-used.patch

Patch1001: 1001-add-as-needed-compile-flags.patch
Patch1002: 1002-optimise-the-shebang-of-the-files-in-python-directly.patch
Patch1003: 1003-remove-hostname-requirement-for-ci-building.patch
Patch1004: 1004-skip-test_task_scheduler_observer.patch
Patch1005: 1005-change-egrep-to-grep-E.patch

BuildRequires: cmake gcc-c++ make
BuildRequires: doxygen
BuildRequires: swig
BuildRequires: python3-devel


%description
Threading Building Blocks (TBB) is a flexible C++ library that simplifies
the work of adding parallelism to complex applications, even if you are
not a threading expert.

The library lets you easily write parallel programs that take full
advantage of the multi-core performance. Such programs are portable,
composable and have a future-proof scalability. oneTBB provides you
with functions, interfaces, and classes to parallelize and scale the
code. All you have to do is to use the templates.


%package devel
Summary: The Threading Building Blocks C++ headers and shared development libraries
Requires: %{name} = %{version}-%{release}

%description devel
The %{name}-devel package contains the deader files and shared object
symlinks for the Threading Building Blocks (TBB) C++ libraries.


%package doc
Summary: The Threading Building Blocks documentation
Requires: %{name} = %{version}-%{release}
Provides: bundled(jquery)

%description doc
PDF documentation for the user of the Threading Building Block (TBB)
C++ library.


%package -n python3-%{name}
Summary: Python 3 TBB module

%description -n python3-%{name}
this package contains the TBB module for python3.

%prep
%autosetup -p1 -n oneTBB-%{version}

# update the arch info in the version_info_linux.sh
sed 's/anolis_arch/%{_arch}/' build/version_info_linux.sh

%build
compiler=""
if [[ %{__cc} == *"gcc"* ]]; then
    compiler="gcc"
elif [[ %{__cc} == *"clang"* ]]; then
    compiler="clang"
else
    compiler="%{__cc}"
fi

%make_build tbb_build_prefix=obj stdver=c++14 \
    compiler=${compiler} \
    CXXFLAGS="%{optflags} -DUSE_PTHREAD" \
    LDFLAGS="$RPM_LD_FLAGS -lpthread"
for file in %{SOURCE6} %{SOURCE7} %{SOURCE8}; do
    base=$(basename ${file})
    sed 's/_FEDORA_VERSION/%{version}/' ${file} > ${base}
    touch -r ${file} ${base}
done

# Build for python 3
. build/obj_release/tbbvars.sh
pushd python
%make_build -C rml stdver=c++14 \
    compiler=${compiler} \
    CPLUS_FLAGS="%{optflags} -DUSE_PTHREAD" \
    LDFLAGS="$RPM_LD_FLAGS -lpthread"
cp -p rml/libirml.so* .
%py3_build
popd

# Build the documentation
%make_build doxygen

%check
make test tbb_build_prefix=obj stdver=c++14 CXXFLAGS="%{optflags}"

%install
mkdir -p $RPM_BUILD_ROOT/%{_libdir}
mkdir -p $RPM_BUILD_ROOT/%{_includedir}

pushd build/obj_release
    for file in libtbb{,malloc{,_proxy}}; do
        install -p -D -m 755 ${file}.so.2 $RPM_BUILD_ROOT/%{_libdir}
        ln -s $file.so.2 $RPM_BUILD_ROOT/%{_libdir}/$file.so
    done
popd

pushd include
    find tbb -type f ! -name \*.htm\* -exec \
        install -p -D -m 644 {} $RPM_BUILD_ROOT/%{_includedir}/{} \
    \;
popd

for file in %{SOURCE6} %{SOURCE7} %{SOURCE8}; do
    install -p -D -m 644 $(basename ${file}) \
        $RPM_BUILD_ROOT/%{_libdir}/pkgconfig/$(basename ${file})
done

# Install the rml headers
mkdir -p $RPM_BUILD_ROOT%{_includedir}/rml
cp -p src/rml/include/*.h $RPM_BUILD_ROOT%{_includedir}/rml

# Python 3 install
. build/obj_release/tbbvars.sh
pushd python
%py3_install
chmod a+x $RPM_BUILD_ROOT%{python3_sitearch}/TBB.py
chmod a+x $RPM_BUILD_ROOT%{python3_sitearch}/tbb/__*.py
cp -p libirml.so.1 $RPM_BUILD_ROOT%{_libdir}
ln -s libirml.so.1 $RPM_BUILD_ROOT%{_libdir}/libirml.so
popd

# Install the cmake files
cmake \
  -DINSTALL_DIR=$RPM_BUILD_ROOT%{_libdir}/cmake/TBB \
  -DSYSTEM_NAME=Linux \
  -DLIB_REL_PATH=../.. \
  -P cmake/tbb_config_installer.cmake

%generate_compatibility_deps
rm -rf $RPM_BUILD_ROOT%{abidir}/_api.cpython-310-*-linux-gnu.dump

%files
%license LICENSE
%dir %{abidir}
%{_libdir}/libtbb.so.2
%{abidir}/libtbb.dump
%{_libdir}/libtbbmalloc.so.2
%{abidir}/libtbbmalloc.dump
%{_libdir}/libtbbmalloc_proxy.so.2
%{abidir}/libtbbmalloc_proxy.dump
%{_libdir}/libirml.so.1
%{abidir}/libirml.dump

%files devel
%{_includedir}/rml/
%{_includedir}/tbb/
%{_libdir}/*.so
%{_libdir}/cmake/TBB/
%{_libdir}/pkgconfig/*.pc

%files -n python3-%{name}
%doc python/index.html
%{python3_sitearch}/TBB*
%{python3_sitearch}/tbb/
%{python3_sitearch}/__pycache__/TBB*

%files doc
%doc doc/Release_Notes.txt README.md
%doc CHANGES cmake/README.rst
%doc html

%changelog
* Fri Apr 14 2023 mgb01105731 <mgb01105731@alibaba-inc.com> - 2020.3-4
- Del Specific Fields

* Thu Apr 13 2023 Kun(llfl) <llfl@linux.alibaba.com> - 2020.3-3
- optimize spec file

* Mon Mar 13 2023 Ziyang Zhang <ZiyangZhang@linux.alibaba.com> - 2020.3-2
- Add abi/api files

* Tue Jul 26 2022 happy_orange <songnannan@linux.alibaba.com> - 2020.3-1
- Init pcakage from upstream
